/* rsa.js: http://www-cs-students.stanford.edu/~tjw/jsbn/ */

// Depends on jsbn.js and rng.js
// Version 1.1: support utf-8 encoding in pkcs1pad2

SYNO = window.SYNO || {};
SYNO.Encryption = SYNO.Encryption || {};
SYNO.Encryption.RSA = (function () {

	// convert a (hex) string to a bignum object


	function parseBigInt(str, r) {
		return new SYNO.Encryption.BigInteger(str, r);
	}

	// PKCS#1 (type 2, random) pad input string s to n bytes, and return a bigint


	function pkcs1pad2(s, n) {
		/* jshint bitwise: false */
		if (n < s.length + 11) { // TODO: fix for utf-8
			//alert("Message too long for RSA");
			return null;
		}
		var ba = [];
		var i = s.length - 1;
		while (i >= 0 && n > 0) {
			var c = s.charCodeAt(i--);
			if (c < 128) { // encode using utf-8
				ba[--n] = c;
			} else if ((c > 127) && (c < 2048)) {
				ba[--n] = (c & 63) | 128;
				ba[--n] = (c >> 6) | 192;
			} else {
				ba[--n] = (c & 63) | 128;
				ba[--n] = ((c >> 6) & 63) | 128;
				ba[--n] = (c >> 12) | 224;
			}
		}
		ba[--n] = 0;
		var rng = new SYNO.Encryption.SecureRandom();
		var x = [];
		while (n > 2) { // random non-zero pad
			x[0] = 0;
			while (x[0] === 0) {
				rng.nextBytes(x);
			}
			ba[--n] = x[0];
		}
		ba[--n] = 2;
		ba[--n] = 0;
		return new SYNO.Encryption.BigInteger(ba);
	}

	// "empty" RSA key constructor


	function RSAKey() {
		this.n = null;
		this.e = 0;
		this.d = null;
		this.p = null;
		this.q = null;
		this.dmp1 = null;
		this.dmq1 = null;
		this.coeff = null;
	}

	// Set the public key fields N and e from hex strings


	function RSASetPublic(N, E) {
		if (!SYNO.isEmpty(N) && !SYNO.isEmpty(E) && N.length > 0 && E.length > 0) {
			this.n = parseBigInt(N, 16);
			this.e = parseInt(E, 16);
		} else {
			//alert("Invalid RSA public key");
		}
	}

	// Perform raw public operation on "x": return x^e (mod n)


	function RSADoPublic(x) {
		return x.modPowInt(this.e, this.n);
	}

	// Return the PKCS#1 RSA encryption of "text" as an even-length hex string


	function RSAEncrypt(text) {
		/* jshint bitwise: false */
		var m = pkcs1pad2(text, (this.n.bitLength() + 7) >> 3);
		if (SYNO.isEmpty(m)) {
			return null;
		}
		var c = this.doPublic(m);
		if (SYNO.isEmpty(c)) {
			return null;
		}
		var h = c.toString(16);
		if ((h.length & 1) === 0) {
			return h;
		} else {
			return "0" + h;
		}
	}

	// Return the PKCS#1 RSA encryption of "text" as a Base64-encoded string
	//function RSAEncryptB64(text) {
	//  var h = this.encrypt(text);
	//  if(h) return hex2b64(h); else return null;
	//}
	// protected
	RSAKey.prototype.doPublic = RSADoPublic;

	// public
	RSAKey.prototype.setPublic = RSASetPublic;
	RSAKey.prototype.encrypt = RSAEncrypt;
	//RSAKey.prototype.encrypt_b64 = RSAEncryptB64;
	return RSAKey;
})();
